from numpy import exp, pi, real, hstack, arange
from scipy import ifft


def chebexp(y):
    """Efficiently compute coefficients $\alpha_j$ in the Chebychev expansion
    $p = \sum_{j=0}^{n} \alpha_j T_j$ of $p \in \Cp_n$ based on values $y_k$,
    $k=0,\ldots,n$, in Chebychev nodes $t_k$, $k=0,\ldots,n$. These values are
    passed in the row vector $y$.
    """
    # degree of polynomial
    n = y.shape[0] -1

    # create r.h.s vector $z$ by wrapping and componentwise scaling
    t = arange(0, 2*n+2)
    z = exp(-pi*1.0j*n/(n+1.0)*t) * hstack([y,y[::-1]])

    # Solve linear system for $\zeta$ with effort $O(n\log n)$
    c = ifft(z)

    # recover $\gamma_j$
    t = arange(-n, n+2)
    b = real(exp(0.5j*pi/(n+1.0)*t) * c)

    # recover coefficients $a$ of Chebyshev expansion
    a = hstack([ b[n], 2*b[n+1:2*n+1] ])

    return a
